@@ -24,7 +24,7 @@ getFont :: FontStyle -> Char -> Font
 getFont fontStyle c
   = fromMaybe GnuUnifont
       ((Data.Map.lookup fontStyle megafont2) >>=
-         return . fromCharToFont . (! (ord c)))
+         return . fromCharToFont . (! (if (ord c) >65535 then ord (' ') else (ord c))))
  
 {-DHUN| Takes a font and returns the LaTeX Command to switch to this particular font in xelatex DHUN-}
  
