@@ -114,8 +114,9 @@ getImages ::
               WikiUrl ->
                 ImperativeMonad [MVar (Maybe ImageInfo)]
 getImages dir images theWikiUrl
-  = liftIO $
-      do ddir <- base dir
-         thetheWikiUrl <- base theWikiUrl
-         iimages <- mapM base ((zip [1 ..] images))
-         (mapM (liftA3 doImage ddir thetheWikiUrl) iimages)
+  = do st<-get
+       liftIO $
+         do ddir <- base (counter st) dir
+            thetheWikiUrl <- base (counter st) theWikiUrl
+            iimages <- mapM (base (counter st)) ((zip [1 ..] images))
+            (mapM (liftA3 (counter st) doImage ddir thetheWikiUrl) iimages)
