@@ -230,7 +230,8 @@ getContributors ::
                     (MVar ([MVar (Map String Contributor)], [MVar (Maybe String)]))
 getContributors u
   = do st <- get
-       put imperativeStateZero
+       stz<- liftIO imperativeStateZero
+       put stz {counter=counter st}
        au <- mapM go u
        newState <- get
        put st
@@ -247,13 +248,13 @@ addContributors ::
 addContributors theLemma uu
   = do sst <- get
        let st = fullUrl sst
-       thetheLemma <- liftIO . base $ theLemma
-       thetheHostname <- liftIO . base . hostname $ st
-       thetheUU <- liftIO . base $ uu
-       au <- (liftIO (((liftA3 fun) thetheLemma thetheHostname thetheUU)))
+       thetheLemma <- liftIO . (base (counter sst)) $ theLemma
+       thetheHostname <- liftIO . (base (counter sst)) . hostname $ st
+       thetheUU <- liftIO . (base (counter sst)) $ uu
+       au <- (liftIO (((liftA3 (counter sst) fun) thetheLemma thetheHostname thetheUU)))
                :: ImperativeMonad (MVar (Map String Contributor, Maybe String))
-       auau <- liftIO ((liftA (return . fst)) au)
-       lic <- liftIO ((liftA (return . snd)) au)
+       auau <- liftIO ((liftA (counter sst) (return . fst)) au)
+       lic <- liftIO ((liftA (counter sst) (return . snd)) au)
        put sst{audict = auau : (audict sst)}
        return lic
   where fun lem ho uuu
