@@ -2,13 +2,14 @@
 module Parallel where
 import Control.Concurrent.MVar
 import Control.Concurrent
+import Control.DeepSeq
  
 {-DHUN| takes a function of return type IO which takes on parameter and an MVar containing at the same type as the parameter of the function and returns and IO action containing an MVar containing the same type as the tyoe contained in the IO Action returned by the function. This function retuns immediately and passed an empty MVar as return type. As soon as the MVar containing the same type as the parameter of the function is ready for reading the the function is executed in a new thread. As soon as the function return a value, the it is written into the returned MVar.  DHUN-}
  
-(<$>) :: (a -> IO b) -> MVar a -> IO (MVar b)
-f <$> x
+(<$>) :: MVar Int -> (a -> IO b) -> MVar a -> IO (MVar b)
+(<$>) vv f x
   = do var <- newEmptyMVar
-       _ <- forkIO (go var)
+       _ <- myFork vv (go var)
        return var
   where go v
           = do xx <- readMVar x
@@ -17,26 +18,39 @@ f <$> x
  
 {-DHUN| takes a value and returns an IO action that contain a filled MVar conatining the value DHUN-}
  
-base :: a -> IO (MVar a)
-base = pure . return
+base :: MVar Int -> a -> IO (MVar a)
+base v x = ((pure v). return) x  
  
 {-DHUN| takes an IO action and return an MVar containing the and IO action the returns an MVar containing the same type as contained by the IO action passed to this function. The IO action passed as first parameter to this function is executed in a new thread. As soon as it finishes its result is written into the returned MVar DHUN-}
  
-pure :: IO a -> IO (MVar a)
-pure x
+pure :: MVar Int ->  IO a -> IO (MVar a)
+pure vv x
   = do var <- newEmptyMVar
-       _ <- forkIO (go var)
+       _ <- myFork vv (go var)
        return var
   where go v
           = do result <- x
                putMVar v result
- 
+
+{-DHUN| alias to switch between forkOS and forkIO for testing DHUN-}
+
+myFork :: MVar Int ->  IO () -> IO ThreadId
+myFork v x = do i<-takeMVar v
+                if i<200 
+                  then do putMVar v $!! (i+1)
+                          forkIO $ do x
+                                      ii<-takeMVar v
+                                      putMVar v $!! (ii-1)
+                  else do putMVar v $!! i
+                          threadDelay 1000
+                          myFork v x
+                        
 {-DHUN| takes a function which takes a list and return an IO action as first parameter. It takes a list of MVar containing the same type as the type contained in the list as second parameter. It returnes an empty MVar immediately. As soon as all MVar in the list given as second parameter could be read, the function is started in a new thread. As soon as it finishes, the returend MVar is filled with the value returend by the function DHUN-}
  
-liftList :: ([a] -> IO b) -> [MVar a] -> IO (MVar b)
-liftList f x
+liftList :: MVar Int -> ([a] -> IO b) -> [MVar a] -> IO (MVar b)
+liftList vv f x
   = do var <- newEmptyMVar
-       _ <- forkIO (go var)
+       _ <- myFork vv (go var)
        return var
   where go v
           = do xx <- mapM readMVar x
@@ -45,10 +59,10 @@ liftList f x
  
 {-DHUN| the same as liftList. Only difference is that the seconds parameter is not a list of MVars containing values but an MVar containing a list of MVars containing values. So an addinal step is made to read the out MVar DHUN-}
  
-liftList2 :: ([a] -> IO b) -> MVar [MVar a] -> IO (MVar b)
-liftList2 f x
+liftList2 :: MVar Int -> ([a] -> IO b) -> MVar [MVar a] -> IO (MVar b)
+liftList2 vv f x
   = do var <- newEmptyMVar
-       _ <- forkIO (go var)
+       _ <- myFork vv (go var)
        return var
   where go v
           = do xxx <- readMVar x
@@ -58,15 +72,15 @@ liftList2 f x
  
 {-DHUN| prefix version of the angle bracked dollar operator described above DHUN-}
  
-liftA :: (a -> IO b) -> MVar a -> IO (MVar b)
-liftA f x = f <$> x
+liftA :: MVar Int ->  (a -> IO b) -> MVar a -> IO (MVar b)
+liftA v f x = (<$>) v f x
  
 {-DHUN| same as liftA. Just function must have exactly two parameter instead of only one DHUN-}
  
-liftA2 :: (a -> b -> IO c) -> MVar a -> MVar b -> IO (MVar c)
-liftA2 f x y
+liftA2 :: MVar Int -> (a -> b -> IO c) -> MVar a -> MVar b -> IO (MVar c)
+liftA2 vv f x y
   = do var <- newEmptyMVar
-       _ <- forkIO (go var)
+       _ <- myFork vv (go var)
        return var
   where go v
           = do xx <- readMVar x
@@ -77,10 +91,10 @@ liftA2 f x y
 {-DHUN| same as liftA. Just function must have exactly three parameter instead of only one DHUN-}
  
 liftA3 ::
-       (a -> b -> c -> IO d) -> MVar a -> MVar b -> MVar c -> IO (MVar d)
-liftA3 f x y z
+       MVar Int -> (a -> b -> c -> IO d) -> MVar a -> MVar b -> MVar c -> IO (MVar d)
+liftA3 vv f x y z
   = do var <- newEmptyMVar
-       _ <- forkIO (go var)
+       _ <- myFork vv (go var)
        return var
   where go v
           = do xx <- readMVar x
