@@ -14,14 +14,14 @@ import Control.Concurrent
 import Control.Monad.State
 import ImperativeState hiding (name)
 import Hex
-import Data.Map
+import Data.Map.Strict
 import Data.Maybe 
 import System.Process hiding (cwd)
 import Data.List
 import Text.Blaze.Internal
 import Data.Time.Clock.POSIX
 import Data.Text (pack)
-
+import Control.DeepSeq
 
 mytext ::String-> H.Html
 mytext = text . pack
@@ -75,10 +75,15 @@ infoBox heading content = H.td $ H.div H.! A.style "padding:10px"  $H.div H.! A.
 
 data ProgressInfo = ProgressInfo { progress :: Double, filename:: Maybe String , startTime::Double, barValue::Double, lastStepTime::Double, lastRuntime::Double, extension::String, failed::Bool}
 
+
+instance NFData ProgressInfo
+  where
+    rnf a = a `seq` ()
+
 {-DHUN| IO action to run the server DHUN-}
  
 serve :: Int -> IO ()
-serve p = do a<-newMVar Data.Map.empty
+serve p = do a<-newMVar Data.Map.Strict.empty
              simpleHTTP nullConf{port = p, timeout = 100000}  $ msum [dirs "progress" $ path $ \subject -> progressBar a subject, dirs "file" $ path $ \subject -> fileFrame a subject, formPage a]
 
 
@@ -88,9 +93,9 @@ serve p = do a<-newMVar Data.Map.empty
  
 template :: Text -> H.Html -> Response
 template title body
-  = toResponse $
-      H.html $
-        do H.head $ do H.title (H.toHtml title)
+  = toResponse $ H.docTypeHtml $
+        do H.head $ do H.meta H.! A.charset "utf-8"
+                       H.title (H.toHtml title)
            H.body $ do body
  
 {-DHUN| takes an url to a wiki article and a filename for the temporary file to be created and return a shell command to run mediawiki2latex to compile a pdf document from the given url and write it to the given temporary filename DHUN-}
@@ -118,7 +123,7 @@ progressBar t sub
                                  ((k,_):_)->k
                                  _ -> 1
                   m<-liftIO (takeMVar t) 
-                  let uu = (fromMaybe progressInfoZero (Data.Map.lookup theIndex m))
+                  let uu = (fromMaybe progressInfoZero (Data.Map.Strict.lookup theIndex m))
                   let nextKnownRelativeProgressToBeReached = gogo mylist (progress uu)
                   let lastReachedKnownRelativeProgress= progress uu
                   let timeOfLastReachedKnownRelativeProgress= lastStepTime uu
@@ -130,7 +135,7 @@ progressBar t sub
                   let oldProgressBarValue = barValue uu
                   let progressBarValue=oldProgressBarValue+(max((p-oldProgressBarValue)*(runtime-(lastRuntime uu))/5.0) 0.0)
                   let prog=if lastReachedKnownRelativeProgress==1.0 then 1000 else if failed uu then  0 else round(progressBarValue*1000.0)::Integer
-                  liftIO $ putMVar t (Data.Map.insert theIndex uu{barValue=progressBarValue,lastRuntime=runtime} m )
+                  liftIO $ putMVar t $!! (Data.Map.Strict.insert theIndex uu{barValue=progressBarValue,lastRuntime=runtime} m )
                   case filename uu of 
                     Nothing -> do method GET 
                                   ok $
@@ -157,7 +162,7 @@ fileFrame t sub
                                  _ -> 1   
                m<-liftIO (takeMVar t) 
                liftIO $ putMVar t m
-               let uu = (fromMaybe progressInfoZero (Data.Map.lookup theIndex m))
+               let uu = (fromMaybe progressInfoZero (Data.Map.Strict.lookup theIndex m))
                case filename uu of 
                  Just x-> do f <- serveFile (guessContentTypeM mimeTypes) x
                              _ <- liftIO  (forkIO (do threadDelay 2000000 
@@ -274,7 +279,7 @@ ex h i e t m n
            then 
             do System.IO.hPutStr i "\n" 
                mm<-takeMVar m
-               putMVar m (Data.Map.update (\x->Just x{failed=True}) n mm)
+               putMVar m (Data.Map.Strict.update (\x->Just x{failed=True}) n mm)
            else return ()
           xi<-hIsOpen i 
           if xi then hFlush i else return ()
@@ -301,12 +306,12 @@ progressDriver n o t s = do xo<-hIsOpen o
                                       else return "" 
                             case msum (Data.List.map (\(k,v)-> if isInfixOf k l then Just v else Nothing) mylist) of
                               Just x -> do m<-takeMVar t 
-                                           putMVar t (Data.Map.insertWith (\new old-> old{progress=progress new,lastStepTime=realToFrac tt}) n progressInfoZero{progress=x} m)
+                                           putMVar t (Data.Map.Strict.insertWith (\new old-> old{progress=progress new,lastStepTime=realToFrac tt}) n progressInfoZero{progress=x} m)
                               _ -> return ()
                             m<-readMVar t
-                            case Data.Map.lookup n m of 
+                            case Data.Map.Strict.lookup n m of 
                               Just yy | (progress yy)==1.0 -> do _<-takeMVar t
-                                                                 putMVar t (Data.Map.insert n yy{filename=Just s} m)
+                                                                 putMVar t (Data.Map.Strict.insert n yy{filename=Just s} m)
                               Just yy | (failed yy)->return ()
                               _ -> progressDriver n o t s 
 progressInfoZero::ProgressInfo 
@@ -319,7 +324,7 @@ forkProgressDriver o t s ext
                               let mm = case (keys m) of 
                                         (x:xs)-> Data.List.maximum (x:xs)
                                         _ -> 0
-                              putMVar t (Data.Map.insert (mm+1) ProgressInfo {progress=0.0,filename=Nothing,startTime=realToFrac tt,barValue=0.0,lastStepTime=0.0,lastRuntime=0.0,extension=ext,failed=False} m)
+                              putMVar t (Data.Map.Strict.insert (mm+1) ProgressInfo {progress=0.0,filename=Nothing,startTime=realToFrac tt,barValue=0.0,lastStepTime=0.0,lastRuntime=0.0,extension=ext,failed=False} m)
                               _<-forkIO (progressDriver (mm+1) o t s)
                               return (mm+1)
 
