@@ -1,6 +1,5 @@
-
+{-DHUN| module to load information on the contributors in images from the wiki website DHUN-}
 module SimpleContributors where
-import Compiler
 import MediaWikiParseTree
 import MediaWikiParser
 import LatexRenderer
