@@ -88,7 +88,7 @@ geturl3 u d s
           let req
                 = (urlEncodedBody
                      (map (\ (a, b) -> (UTF8Str.fromString a, UTF8Str.fromString b))
-                        [("input", d), ("removecomments", "1"), ("removenowiki", "1"),
+                        [("wpInput", d), ("removecomments", "1"), ("removenowiki", "1"),
                          ("generate_xml", "0"), ("contexttitle", s)]))
                     req0
           res <- withManager $ httpLbs req
