@@ -174,7 +174,7 @@ header = "Usage: mediawiki2latex [OPTION...]"
  
 versionHeader :: String
 versionHeader
-  = "mediawiki2latex version 7.14\n" ++ (usageInfo header options)
+  = "mediawiki2latex version 7.19\n" ++ (usageInfo header options)
  
 {-DHUN| print the version string of mediawiki2latex. Takes the output of the compilerOpts function as input. Prints the version string if no options were given or the version command was given does noting otherwise DHUN-}
  
@@ -370,6 +370,7 @@ main
   = do a <- getArgs
        o <- compilerOpts a
        printVersion o
+       stz <- imperativeStateZero
        cwd <- getCurrentDirectory
        case (checkOpts cwd (fst o)) of
            Right x -> case (selfTest x) of
@@ -377,7 +378,7 @@ main
                        _-> case (server x) of
                             Just zz -> serve zz
                             _ -> do (xx, _) <- (runStateT (runErrorT (All.all x))
-                                                imperativeStateZero)
+                                                stz)
                                     case xx of
                                       Left n -> print n
                                       _ -> return ()
